package com.web.api.utils;

import com.aliyun.oss.OSSClient;
import com.aliyun.oss.model.ObjectMetadata;
import com.aliyun.oss.model.PutObjectResult;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.net.URL;
import java.util.Date;
import java.util.Random;

/**
 * @author 王玉鹏
 * @version 1.0
 * @className OssClientUtils
 * @description TODO
 * @date 2019/6/11 13:06
 */
@Slf4j
@Configuration
public class OssClientUtils {

    @Value("${oss.url}")
    private String ossUrl;
    @Value("${oss.bucketName}")
    private String bucketName;
    @Value("${oss.key}")
    private String key;
    @Value("${oss.secret}")
    private String secret;

    private String  imageDir;

    private OSSClient ossClient;

    @Bean
    public OSSClient ossClient(){
        ossClient = new OSSClient(ossUrl, key, secret);

        return ossClient;
    }
    /**
     * 销毁
     */
    public void destory() {
        ossClient.shutdown();
    }

    /**
     * 上传图片 直接获取本地资源路径
     *
     * @param url
     * @throws Exception
     */
    public void uploadImg2Oss(String url,String imageDir) throws Exception {
        this.imageDir=imageDir;
        File fileOnServer = new File(url);
        FileInputStream fin;
        try {
            fin = new FileInputStream(fileOnServer);
            String[] split = url.split("/");
            this.uploadFile2OSS(fin, split[split.length - 1],imageDir);
        } catch (FileNotFoundException e) {
            throw new Exception("图片上传失败");
        }
    }

    /**
     * 上传图片
     * @param file
     * @return    key   可根据key获取上到到服务器的资源和删除等操作
     * @throws Exception
     */
    public String uploadImg2Oss(MultipartFile file, String imageDir) throws Exception {
//    if (file.getSize() > 1024 * 1024) {
//      throw new Exception("上传图片大小不能超过1M！");
//    }
        this.imageDir=imageDir;
        String originalFilename = file.getOriginalFilename();
        String substring = originalFilename.substring(originalFilename.lastIndexOf(".")).toLowerCase();
        Random random = new Random();
        String name = random.nextInt(10000) + System.currentTimeMillis() + substring;
        try {
            InputStream inputStream = file.getInputStream();
            this.uploadFile2OSS(inputStream, name,imageDir);
            return name;
        } catch (Exception e) {
            throw new Exception("图片上传失败");
        }
    }


    /**
     * 上传到OSS服务器  如果同名文件会覆盖服务器上的
     *
     * @param instream 文件流
     * @param fileName 文件名称 包括后缀名
     * @return 出错返回"" ,唯一MD5数字签名
     */
    public String uploadFile2OSS(InputStream instream, String fileName ,String imageDir) {
        this.imageDir=imageDir;
        String ret = "";
        try {
            //创建上传Object的Metadata
            ObjectMetadata objectMetadata = new ObjectMetadata();
            objectMetadata.setContentLength(instream.available());
            objectMetadata.setCacheControl("no-cache");
            objectMetadata.setHeader("Pragma", "no-cache");
            objectMetadata.setContentType(getcontentType(fileName.substring(fileName.lastIndexOf("."))));
            objectMetadata.setContentDisposition("attachment;filename=" + fileName);
            // 设置URL过期时间为10年  3600* 1000*24*365*10
            Date expiration = new Date(System.currentTimeMillis() + 3600L * 1000L * 24L * 365L * 10L);
            objectMetadata.setExpirationTime(expiration);
            PutObjectResult putResult = ossClient.putObject(bucketName,imageDir+fileName, instream, objectMetadata);
            ret = putResult.getETag();
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        } finally {
            try {
                if (instream != null) {
                    instream.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return ret;
    }

    /**
     * 获得图片路径
     *
     * @param fileUrl
     * @return
     */
    public String getImgUrl(String fileUrl) {
        if (!StringUtils.isEmpty(fileUrl)) {
            String[] split = fileUrl.split("/");
            return this.getUrl(split[split.length - 1]);
        }
        return null;
    }



    /**
     * Description: 判断OSS服务文件上传时文件的contentType
     *
     * @param FilenameExtension 文件后缀
     * @return String
     */
    public static String getcontentType(String FilenameExtension) {
        if (FilenameExtension.equalsIgnoreCase(".bmp")) {
            return "image/bmp";
        }
        if (FilenameExtension.equalsIgnoreCase(".gif")) {
            return "image/gif";
        }
        if (FilenameExtension.equalsIgnoreCase(".jpeg") ||
                FilenameExtension.equalsIgnoreCase(".jpg") ||
                FilenameExtension.equalsIgnoreCase(".png")) {
//            return "image/jpeg";
            return "image/jpg";
        }
        if (FilenameExtension.equalsIgnoreCase(".html")) {
            return "text/html";
        }
        if (FilenameExtension.equalsIgnoreCase(".txt")) {
            return "text/plain";
        }
        if (FilenameExtension.equalsIgnoreCase(".vsd")) {
            return "application/vnd.visio";
        }
        if (FilenameExtension.equalsIgnoreCase(".pptx") ||
                FilenameExtension.equalsIgnoreCase(".ppt")) {
            return "application/vnd.ms-powerpoint";
        }
        if (FilenameExtension.equalsIgnoreCase(".docx") ||
                FilenameExtension.equalsIgnoreCase(".doc")) {
            return "application/msword";
        }
        if (FilenameExtension.equalsIgnoreCase(".xml")) {
            return "text/xml";
        }
        if (FilenameExtension.equalsIgnoreCase(".pdf")) {
            return "application/pdf";
        }
        if (FilenameExtension.equalsIgnoreCase(".xlsx")) {
            return "application/vnd.ms-excel";
        }
        return "image/jpeg";
    }

    /**
     * 获得url链接
     *
     * @param key
     * @return
     */
    public String getUrl(String key) {
        // 设置URL过期时间为10年  3600l* 1000*24*365*10
        Date expiration = new Date(System.currentTimeMillis() + 3600L * 1000L * 24L * 365L * 10L);
        // 生成URL
        URL url = ossClient.generatePresignedUrl(bucketName, imageDir+key, expiration);
        if (url != null) {
            return url.toString();
        }
        return null;
    }

    /**
     * 删除单个文件
     */
    public void delFile(String key){
        ossClient.deleteObject(bucketName, key);
    }
}
